unit MainFrm;

{$IFDEF FPC}
{$MODE Delphi}
{$ENDIF}

interface

uses
  {$IFNDEF FPC}
  Windows, Messages,
  {$ELSE}
  LCLIntf, LCLType, FileUtil,
  {$ENDIF}
  SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, MRVSender, MRVCamMultiView, StdCtrls, MRVType,
  MRVMicrophoneViewer, MRVMicrophone, MRVCamera, MRVReceiver, ComCtrls,
  ExtCtrls, MRVWinMicrophone, MRVCamView;

type
  TfrmMain = class(TForm)
    RVCamSender1: TRVCamSender;
    RVCamReceiver1: TRVCamReceiver;
    RVMicrophone1: TRVMicrophone;
    StatusBar1: TStatusBar;
    gbMessages: TGroupBox;
    gbMySound: TGroupBox;
    cbMicrophone: TCheckBox;
    btnMicrophoneOptions: TButton;
    og: TOpenDialog;
    mMessages: TMemo;
    RVCamera1: TRVCamera;
    RVCamMultiView1: TRVCamMultiView;
    cmbConnectionType: TComboBox;
    Label4: TLabel;
    nb: TNotebook;
    GroupBox1: TGroupBox;
    Label5: TLabel;
    Label7: TLabel;
    txtRemoteAddress: TEdit;
    btnConnect: TButton;
    txtRemotePort: TEdit;
    GroupBox2: TGroupBox;
    Label6: TLabel;
    Label8: TLabel;
    txtMyAddress: TEdit;
    txtMyPort: TEdit;
    btnListen: TButton;
    btnSendCmd: TButton;
    btnSendFile: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure RVCamReceiver1SessionConnected(Sender: TObject;
      SessionKey: TRVSessionKey);
    procedure RVCamReceiver1SessionDisconnected(Sender: TObject;
      SessionKey: TRVSessionKey);
    procedure cbMicrophoneClick(Sender: TObject);
    procedure btnMicrophoneOptionsClick(Sender: TObject);
    procedure btnSendFileClick(Sender: TObject);
    procedure btnListenClick(Sender: TObject);
    procedure btnSendCmdClick(Sender: TObject);
    procedure RVCamReceiver1ReceiveCmdData(Sender: TObject;
      SessionKey: TRVSessionKey; Cmd: TRVCmd; ASocket: TRVSocket; nGUIDFrom,
      nGUIDTo, nGUIDGroup: TGUID; AMediaIndex: Word);
    procedure RVCamReceiver1ReceivedFile(Sender: TObject;
      SessionKey: TRVSessionKey; FileName: string; TotalFileSize: Int64;
      ASocket: TRVSocket; AGUIDFrom, AGUIDTo, AGUIDGroup: TGUID);
    procedure RVCamReceiver1ReceiveFileData(Sender: TObject;
      SessionKey: TRVSessionKey; FileName: string; FileOffs,
      TotalFileSize: Int64; AData: TStream; ASocket: TRVSocket; AGUIDFrom,
      AGUIDTo, AGUIDGroup: TGUID; AMediaIndex: Word);
    procedure cmbConnectionTypeChange(Sender: TObject);
  private
    { Private declarations }
    FFilePath:                 String;
    FConnecting:               Boolean;
    FCurrentSessionKey:        TRVSessionKey;
  public
    { Public declarations }
    procedure StatusChange(Camera: TRVCamera; const Status: string);
  end;

var
  frmMain: TfrmMain;

implementation

uses AudioOptionsFrm;

{$R *.dfm}

{================================= TfrmMain ===================================}
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  txtRemoteAddress.Text := String(GetLocalHostAddress);
  nb.PageIndex := 0;

  Caption := '[Not Connected]';

  FFilePath := ExtractFilePath(Application.ExeName) + 'Files-' +
    String(RVCamReceiver1.GUIDMy) + {$IFDEF UNIX}'/'{$ELSE}'\'{$ENDIF};

  txtMyAddress.Text := String(GetLocalHostAddress);

  // Setting up a video filter.
  if RVCamReceiver1.Senders.Count = 0 then
    RVCamReceiver1.Senders.Add;
  if RVCamReceiver1.Senders[0].VideoSenders.Count = 0 then
    RVCamReceiver1.Senders[0].VideoSenders.Add;

  RVCamera1.URL := 'http://www.warwick.ac.uk/newwebcam/cgi-bin/webcam.pl';
  RVCamera1.PlayVideoStream;
end;

{------------------------------------------------------------------------------}
// Connecting or disconnecting (on button click)
procedure TfrmMain.btnConnectClick(Sender: TObject);
var
  RemoteSenderPort   : Word;
  RemoteReceiverPort : Word;
begin
  if FConnecting or (RVCamReceiver1.State = rvmsConnecting) or
    (RVCamReceiver1.State = rvmsDisconnecting) then
    exit;

  FConnecting := True;
  try
    btnConnect.Enabled := False;
    if not RVCamReceiver1.Active then
    begin
      // Our receiver will connect to remote sender
      RemoteSenderPort   := StrToInt(txtRemotePort.Text);
      // Our sender will connect to remote receiver
      RemoteReceiverPort := RemoteSenderPort + 1;

      // setting active connection role: the both our sender and receiver
      // will connect to remote counterparts
      RVCamSender1.TCPConnectionType   := rvtcpSenderToReceiver;
      RVCamReceiver1.TCPConnectionType := rvtcpReceiverToSender;

      RVCamSender1.ReceiverHost := txtRemoteAddress.Text;
      RVCamSender1.ReceiverPort := RemoteReceiverPort;

      RVCamReceiver1.Senders.Items[0].SenderHost :=
        txtRemoteAddress.Text;
      RVCamReceiver1.Senders.Items[0].SenderPort :=
        RemoteSenderPort;
      StatusBar1.SimpleText := 'Connecting...';
    end;

    RVCamSender1.Active   := not RVCamSender1.Active;
    RVCamReceiver1.Active := not RVCamReceiver1.Active;

    if RVCamReceiver1.Active then
      btnConnect.Caption := 'Dis&connect'
    else
      btnConnect.Caption := '&Connect';

  finally
    FConnecting := False;
  end;
end;
{------------------------------------------------------------------------------}
{ Preparing to listen (waiting for connection) }
procedure TfrmMain.btnListenClick(Sender: TObject);
var
  IsActive : boolean;
  MySenderPort   : Word;
  MyReceiverPort : Word;
begin
  if FConnecting then
    exit;

  FConnecting := True;
  try
    btnListen.Enabled   := False;

    if not RVCamSender1.Active then
    begin
      // we use two ports:
      // - specified by the user: for our sender
      MySenderPort   := StrToInt(txtMyPort.Text);
      // - specified by the user + 1: for our receiver
      MyReceiverPort := MySenderPort + 1;

      RVCamSender1.TCPConnectionType   := rvtcpReceiverToSender;
      RVCamReceiver1.TCPConnectionType := rvtcpSenderToReceiver;

      RVCamSender1.SenderPort := MySenderPort;
      RVCamReceiver1.Port     := MyReceiverPort;
    end;
    RVCamSender1.Active   := not RVCamSender1.Active;
    RVCamReceiver1.Active := not RVCamReceiver1.Active;

    if RVCamSender1.Active then
      btnListen.Caption := 'Not &listen'
    else
      btnListen.Caption := '&Listen';

    btnListen.Enabled   := True;

    IsActive := RVCamSender1.Active and RVCamReceiver1.Active;
    btnSendCmd.Enabled  := IsActive;
    btnSendFile.Enabled := IsActive;
  finally
    FConnecting := False;
  end;
end;
{------------------------------------------------------------------------------}
// Event: we've received some command
procedure TfrmMain.RVCamReceiver1ReceiveCmdData(Sender: TObject;
  SessionKey: TRVSessionKey; Cmd: TRVCmd; ASocket: TRVSocket; nGUIDFrom,
  nGUIDTo, nGUIDGroup: TGUID; AMediaIndex: Word);
begin
  mMessages.Lines.Add('CMD: ' + String(Cmd.Cmd));
end;
{------------------------------------------------------------------------------}
// Event: we've received a file
procedure TfrmMain.RVCamReceiver1ReceivedFile(Sender: TObject;
  SessionKey: TRVSessionKey; FileName: string; TotalFileSize: Int64;
  ASocket: TRVSocket; AGUIDFrom, AGUIDTo, AGUIDGroup: TGUID);
begin
  mMessages.Lines.Add('File has been received: '+ FileName);
end;
{------------------------------------------------------------------------------}
// Event: receiving a file
procedure TfrmMain.RVCamReceiver1ReceiveFileData(Sender: TObject;
  SessionKey: TRVSessionKey; FileName: string; FileOffs, TotalFileSize: Int64;
  AData: TStream; ASocket: TRVSocket; AGUIDFrom, AGUIDTo, AGUIDGroup: TGUID;
  AMediaIndex: Word);
var
  FileStream: TFileStream;
begin
  FileName := FFilePath + FileName;
  if FileOffs > 0 then
    FileStream := TFileStream.Create(FileName, fmOpenWrite)
  else
  begin
    {$IFDEF FPC}
    CreateDir(FFilePath);
    {$ELSE}
    CreateDirectory(PChar(FFilePath), nil);
    {$ENDIF}
    FileStream := TFileStream.Create(FileName, fmCreate);
  end;
  try
    FileStream.Position := FileOffs;
    FileStream.CopyFrom(AData, 0);
  finally
    FileStream.Free;
  end;
end;
{------------------------------------------------------------------------------}
// Event: connected
procedure TfrmMain.RVCamReceiver1SessionConnected(Sender: TObject;
  SessionKey: TRVSessionKey);
begin
  FCurrentSessionKey := RVCamReceiver1.SessionKey;
  StatusBar1.SimpleText := 'Connected to the server';
  Caption := '[Connected]';

  btnConnect.Enabled := True;

  btnSendCmd.Enabled  := True;
  btnSendFile.Enabled := True;
end;
{------------------------------------------------------------------------------}
// Event: disconnected
procedure TfrmMain.RVCamReceiver1SessionDisconnected(Sender: TObject;
  SessionKey: TRVSessionKey);
begin
  FCurrentSessionKey := 0;

  RVCamSender1.Active := False;
  RVCamReceiver1.Active := False;

  StatusBar1.SimpleText := 'Disconnected from the server';
  btnConnect.Enabled := True;
  Caption := '[Not Connected]';

  btnSendCmd.Enabled  := False;
  btnSendFile.Enabled := False;
end;

{------------------------------------------------------------------------------}
// Turning microphone on/off
procedure TfrmMain.cbMicrophoneClick(Sender: TObject);
begin
  RVMicrophone1.Active := cbMicrophone.Checked;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.cmbConnectionTypeChange(Sender: TObject);
begin
  nb.PageIndex:= cmbConnectionType.ItemIndex;
end;
{------------------------------------------------------------------------------}
// Showing a microphone options form
procedure TfrmMain.btnMicrophoneOptionsClick(Sender: TObject);
begin
  frmAudioOptions.Init(RVMicrophone1);
  frmAudioOptions.Show;
end;
{------------------------------------------------------------------------------}
// Sending a test command
procedure TfrmMain.btnSendCmdClick(Sender: TObject);
var
  Cmd: TRVCmd;
begin
  if RVCamSender1.SessionKey = 0 then
    exit;

  Cmd := TRVCmd.Create;
  Cmd.Cmd := 'MY_CMD';
  RVCamSender1.SendCmd(Cmd);

  RVCamSender1.WaitForSendCmd(True);
end;
{------------------------------------------------------------------------------}
// Sending a file
procedure TfrmMain.btnSendFileClick(Sender: TObject);
begin
  if og.Execute then
    RVCamSender1.SendFile(og.FileName, 0);
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.StatusChange(Camera: TRVCamera; const Status: string);
begin
  if StatusBar1 <> nil then
    StatusBar1.SimpleText := Status;
end;



end.
